<?php

namespace Drupal\commerce_from_price\Plugin\Field\FieldFormatter;

use Drupal\commerce\Context;
use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_price\Plugin\Field\FieldFormatter\PriceCalculatedFormatter;
use Drupal\commerce_price\Price;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Language\LanguageInterface;

/**
 * Plugin implementation of the 'commerce_from_price_calculated' formatter.
 *
 * @FieldFormatter(
 *   id = "commerce_from_price_calculated",
 *   label = @Translation("Calculated 'from' price"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class FromPriceCalculatedFormatter extends PriceCalculatedFormatter {

  use FromPriceFormatterTrait;

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    $entities = $this->getPurchasableEntities($items);
    if (!$entities) {
      return $element;
    }

    $entity = $this->getLowestPricedEntity($entities);
    $lowest_price = $this->getResolvedPrice($entity);

    // If requested, consider multiple equal prices as a single price.
    $multiple = !($this->validateEqualPrices($entities) && $this->getSetting('process_equal_prices_as_single')) && $items->count() > 1;

    // Display the lowest price with the appropriate currency.
    $element = [
      '#theme' => 'commerce_from_price',
      '#label_before' => $multiple
        ? $this->getSetting('multiple_entities')['label_before']
        : $this->getSetting('single_entity')['label_before'],
      '#price' => [
        '#theme' => 'commerce_price_calculated',
        '#calculated_price' => $this->currencyFormatter->format(
          $lowest_price->getNumber(),
          $lowest_price->getCurrencyCode(),
          $this->getFormattingOptions()
        ),
        '#purchasable_entity' => $entity,
        '#cache' => [
          'tags' => $entity->getCacheTags(),
          'contexts' => Cache::mergeContexts($entity->getCacheContexts(), [
            'languages:' . LanguageInterface::TYPE_INTERFACE,
            'country',
          ]),
        ],
      ],
      '#label_after' => $multiple
        ? $this->getSetting('multiple_entities')['label_after']
        : $this->getSetting('single_entity')['label_after'],
      '#cache' => [
        'contexts' => [
          'languages:' . LanguageInterface::TYPE_INTERFACE,
          'country',
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getResolvedPrice(PurchasableEntityInterface $entity): Price {
    $context = new Context($this->currentUser, $this->currentStore->getStore(), NULL, [
      'field_name' => $entity->get('price')->getName(),
    ]);

    return $this->chainPriceResolver->resolve($entity, 1, $context);
  }

}
