<?php

namespace Drupal\commerce_from_price\Plugin\Field\FieldFormatter;

use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_price\Price;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Trait containing logic for all 'commerce_from_price' formatters.
 */
trait FromPriceFormatterTrait {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'single_entity' => [
        'label_before' => '',
        'label_after' => '',
      ],
      'multiple_entities' => [
        'label_before' => new TranslatableMarkup('Starting at'),
        'label_after' => '',
      ],
      'process_equal_prices_as_single' => 1,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['process_equal_prices_as_single'] = [
      '#type' => 'checkbox',
      '#title' => new TranslatableMarkup('Consider purchasable entities with equal prices as single purchasable entity'),
      '#default_value' => $this->getSetting('process_equal_prices_as_single'),
    ];

    $elements['single_entity'] = [
      '#type' => 'details',
      '#title' => new TranslatableMarkup('Single purchasable entity'),
    ];

    $elements['single_entity']['label_before'] = [
      '#type' => 'textfield',
      '#title' => new TranslatableMarkup('Label before'),
      '#default_value' => $this->getSetting('single_entity')['label_before'],
      '#description' => new TranslatableMarkup('Label displayed before the price.'),
    ];

    $elements['single_entity']['label_after'] = [
      '#type' => 'textfield',
      '#title' => new TranslatableMarkup('Label after'),
      '#default_value' => $this->getSetting('single_entity')['label_after'],
      '#description' => new TranslatableMarkup('Label displayed after the price.'),
    ];

    $elements['multiple_entities'] = [
      '#type' => 'details',
      '#title' => new TranslatableMarkup('Multiple purchasable entities'),
    ];

    $elements['multiple_entities']['label_before'] = [
      '#type' => 'textfield',
      '#title' => new TranslatableMarkup('Label before'),
      '#default_value' => $this->getSetting('multiple_entities')['label_before'],
      '#description' => new TranslatableMarkup('Label displayed before the price.'),
    ];

    $elements['multiple_entities']['label_after'] = [
      '#type' => 'textfield',
      '#title' => new TranslatableMarkup('Label after'),
      '#default_value' => $this->getSetting('multiple_entities')['label_after'],
      '#description' => new TranslatableMarkup('Label displayed after the price.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $summary[] = new TranslatableMarkup('Consider equal prices as single entity price: @value', [
      '@value' => $this->getSetting('process_equal_prices_as_single'),
    ]);

    $summary[] = Markup::create('<strong>' . new TranslatableMarkup('Single purchasable entity') . '</strong>');

    $summary[] = new TranslatableMarkup('Label before: @value', [
      '@value' => $this->getSetting('single_entity')['label_before'],
    ]);

    $summary[] = new TranslatableMarkup('Label after: @value', [
      '@value' => $this->getSetting('single_entity')['label_after'],
    ]);

    $summary[] = Markup::create('<strong>' . new TranslatableMarkup('Multiple purchasable entities') . '</strong>');

    $summary[] = new TranslatableMarkup('Label before: @value', [
      '@value' => $this->getSetting('multiple_entities')['label_before'],
    ]);

    $summary[] = new TranslatableMarkup('Label after: @value', [
      '@value' => $this->getSetting('multiple_entities')['label_after'],
    ]);

    return $summary;
  }

  /**
   * Get published purchasable entities from the field item list.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The field item list.
   *
   * @return \Drupal\commerce\PurchasableEntityInterface[]
   *   The purchasable entities.
   */
  public function getPurchasableEntities(FieldItemListInterface $items): array {
    $entities = [];

    foreach ($items as $item) {
      /** @var \Drupal\commerce\PurchasableEntityInterface $entity */
      $entity = $item->get('entity')->getTarget()->getEntity();
      if ($entity->isPublished()) {
        $entities[] = $entity;
      }
    }

    return $entities;
  }

  /**
   * Check if entities have equal prices.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface[] $entities
   *   The entities to compare.
   *
   * @return bool
   *   True if all prices are equal for given entities.
   */
  public function validateEqualPrices(array $entities): bool {
    $previous_price = NULL;

    foreach ($entities as $entity) {
      $price = $this->getResolvedPrice($entity);
      if (NULL !== $previous_price && $price->getNumber() !== $previous_price->getNumber()) {
        return FALSE;
      }

      $previous_price = $price;
    }

    return TRUE;
  }

  /**
   * Get the lowest priced entity.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface[] $entities
   *   The entities to compare.
   *
   * @return \Drupal\commerce\PurchasableEntityInterface|null
   *   The entity with the lowest price or null if input was empty.
   */
  public function getLowestPricedEntity(array $entities): ?PurchasableEntityInterface {
    $lowest_priced_entity = NULL;

    foreach ($entities as $entity) {
      $price = $this->getResolvedPrice($entity);
      if ($lowest_priced_entity === NULL || $price->getNumber() < $lowest_priced_entity->getPrice()->getNumber()) {
        $lowest_priced_entity = $entity;
      }
    }

    return $lowest_priced_entity;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition): bool {
    $entity_type = $field_definition->getTargetEntityTypeId();
    $field_name = $field_definition->getName();

    return $entity_type === 'commerce_product' && $field_name === 'variations';
  }

  /**
   * Get resolved price of purchasable entity.
   *
   * @param \Drupal\commerce\PurchasableEntityInterface $entity
   *   The entity to calculate the price for.
   *
   * @return \Drupal\commerce_price\Price
   *   The price.
   */
  public function getResolvedPrice(PurchasableEntityInterface $entity): Price {
    return $entity->getPrice();
  }

}
