<?php

namespace Drupal\commerce_from_price\Plugin\Field\FieldFormatter;

use Drupal\commerce\Context;
use Drupal\commerce\PurchasableEntityInterface;
use Drupal\commerce_order\Plugin\Field\FieldFormatter\PriceCalculatedFormatter;
use Drupal\commerce_price\Price;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Language\LanguageInterface;

/**
 * Alternative implementation of the 'commerce_from_price_calculated' formatter.
 *
 * This replaces the original commerce_from_price_calculated class when
 * commerce_order module is installed.
 */
class OrderFromPriceCalculatedFormatter extends PriceCalculatedFormatter {

  use FromPriceFormatterTrait;

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    $entities = $this->getPurchasableEntities($items);
    if (!$entities) {
      return $element;
    }

    /** @var \Drupal\commerce\PurchasableEntityInterface $entity */
    $entity = $this->getLowestPricedEntity($entities);
    $context = new Context($this->currentUser, $this->currentStore->getStore(), NULL, [
      'field_name' => $entity->get('price')->getName(),
    ]);
    $adjustment_types = array_filter($this->getSetting('adjustment_types'));
    $result = $this->priceCalculator->calculate($entity, 1, $context, $adjustment_types);
    $calculated_price = $result->getCalculatedPrice();

    // If requested, consider multiple equal prices as a single entity price.
    $multiple = !($this->validateEqualPrices($entities) && $this->getSetting('process_equal_prices_as_single')) && $items->count() > 1;

    // Display the lowest price with the appropriate currency.
    $element = [
      '#theme' => 'commerce_from_price',
      '#label_before' => $multiple
        ? $this->getSetting('multiple_entities')['label_before']
        : $this->getSetting('single_entity')['label_before'],
      '#price' => [
        '#theme' => 'commerce_price_calculated',
        '#result' => $result,
        '#calculated_price' => $this->currencyFormatter->format($calculated_price->getNumber(), $calculated_price->getCurrencyCode(), $this->getFormattingOptions()),
        '#base_price' => $this->currencyFormatter->format($result->getBasePrice()->getNumber(), $calculated_price->getCurrencyCode(), $this->getFormattingOptions()),
        '#adjustments' => $result->getAdjustments(),
        '#purchasable_entity' => $entity,
        '#cache' => [
          'tags' => $entity->getCacheTags(),
          'contexts' => Cache::mergeContexts($entity->getCacheContexts(), [
            'languages:' . LanguageInterface::TYPE_INTERFACE,
            'country',
          ]),
        ],
      ],
      '#label_after' => $multiple
        ? $this->getSetting('multiple_entities')['label_after']
        : $this->getSetting('single_entity')['label_after'],
      '#cache' => [
        'contexts' => [
          'languages:' . LanguageInterface::TYPE_INTERFACE,
          'country',
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getResolvedPrice(PurchasableEntityInterface $entity): Price {
    $context = new Context($this->currentUser, $this->currentStore->getStore(), NULL, [
      'field_name' => $entity->get('price')->getName(),
    ]);

    $adjustment_types = array_filter($this->getSetting('adjustment_types'));
    $result = $this->priceCalculator->calculate($entity, 1, $context, $adjustment_types);

    return $result->getCalculatedPrice();
  }

}
